from jinja2 import Template
from promptflow.core import tool
from promptflow.connections import CustomConnection
from promptflow.contracts.types import PromptTemplate
from .AIF_client import AIFLLMClient


@tool
def custom_llm(
    connection: CustomConnection, 
    api: str,          
    model_name: str,
    temperature: float,
    prompt: PromptTemplate,
    top_p: float,
    max_tokens: int,
    response_format: object,
    presence_penalty: float,
    frequency_penalty: int,    
    **kwargs
) -> str:  
    client = AIFLLMClient(
        auth_url=connection.configs['auth_url'],
        client_id=connection.configs['client_id'],
        client_secret=connection.secrets['client_secret'],
        scope=connection.configs['scope'],
        subscription_key=connection.configs['subscription_key']
    )   
    
    # Replace with your tool code, customise your own code to handle and use the prompt here.
    # Usually connection contains configs to connect to an API.
    # Not all tools need a connection. You can remove it if you don't need it.

    apiURL=f"{connection.configs['api_url']}/{model_name}"
    rendered_prompt = Template(prompt, trim_blocks=True, keep_trailing_newline=True).render(**kwargs)
    response = client.call_custom_model_api(
        apiURL= apiURL,       
        input_prompt=rendered_prompt,         
        temperature=temperature,
        top_p=top_p,
        max_tokens=max_tokens,
        response_format=response_format,
        presence_penalty=presence_penalty,
        frequency_penalty=frequency_penalty
    )
    return response


import requests
import json

class AIFLLMClient:
    def __init__(self, auth_url, client_id, client_secret, scope, subscription_key):
        self.auth_url = auth_url
        self.client_id = client_id
        self.client_secret = client_secret
        self.scope = scope
        self.subscription_key = subscription_key
        self.token = self.get_access_token()

    def get_access_token(self):
        response = requests.post(
            self.auth_url,
            data={"grant_type": "client_credentials", "scope": self.scope},
            auth=(self.client_id, self.client_secret),
        )
        response.raise_for_status()
        return response.json()['access_token']

    def call_custom_model_api(self, apiURL, input_prompt, temperature, top_p, max_tokens, response_format, presence_penalty, frequency_penalty):
        headers = {
            "Content-Type": "application/x-www-form-urlencoded",
            "Authorization": f"Bearer {self.token}",
            "Ocp-Apim-Subscription-Key": self.subscription_key
        }

        input_body = {
            "messages": [
                {
                    "role": "user",
                    "content": input_prompt
                }
            ],
            "temperature": temperature,
            "top_p": top_p,
            "max_tokens": max_tokens,
            "presence_penalty": presence_penalty,
            "frequency_penalty": frequency_penalty
        }

        payload = json.dumps(input_body)
        response = requests.post(apiURL, data=payload, headers=headers, verify=False)
        response.raise_for_status()
        return response.json()

    def run_embedding_model(self, url, input_text):
        headers = {
            'Ocp-Apim-Subscription-Key': self.subscription_key,
            'Authorization': f'Bearer {self.token}',
            'Content-Type': 'application/x-www-form-urlencoded',
        }
        data = json.dumps({"input": input_text})
        response = requests.post(url, headers=headers, data=data, verify=False)
        response.raise_for_status()
        return response.json()